@@ -15,7 +15,7 @@ use {
     LinalgScalar,
 };
 
-use std::any::{Any, TypeId};
+use std::any::TypeId;
 
 #[cfg(feature="blas")]
 use std::cmp;
@@ -40,7 +40,7 @@ const GEMM_BLAS_CUTOFF: usize = 7;
 type blas_index = c_int; // blas index type
 
 
-impl<A, S> ArrayBase<S, Ix>
+impl<A, S> ArrayBase<S, Ix1>
     where S: Data<Elem=A>,
 {
     /// Compute the dot product of one-dimensional arrays.
@@ -49,14 +49,14 @@ impl<A, S> ArrayBase<S, Ix>
     /// of complex operands, and thus not their inner product).
     ///
     /// **Panics** if the arrays are not of the same length.
-    pub fn dot<S2>(&self, rhs: &ArrayBase<S2, Ix>) -> A
+    pub fn dot<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
         where S2: Data<Elem=A>,
               A: LinalgScalar,
     {
         self.dot_impl(rhs)
     }
 
-    fn dot_generic<S2>(&self, rhs: &ArrayBase<S2, Ix>) -> A
+    fn dot_generic<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
         where S2: Data<Elem=A>,
               A: LinalgScalar,
     {
@@ -77,7 +77,7 @@ impl<A, S> ArrayBase<S, Ix>
     }
 
     #[cfg(not(feature="blas"))]
-    fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix>) -> A
+    fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
         where S2: Data<Elem=A>,
               A: LinalgScalar,
     {
@@ -85,7 +85,7 @@ impl<A, S> ArrayBase<S, Ix>
     }
 
     #[cfg(feature="blas")]
-    fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix>) -> A
+    fn dot_impl<S2>(&self, rhs: &ArrayBase<S2, Ix1>) -> A
         where S2: Data<Elem=A>,
               A: LinalgScalar,
     {
@@ -201,7 +201,7 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix2>> for ArrayBase<S, Ix2>
         let b = b.view();
         let ((m, k), (k2, n)) = (a.dim(), b.dim());
         if k != k2 || m.checked_mul(n).is_none() {
-            return dot_shape_error(m, k, k2, n);
+            dot_shape_error(m, k, k2, n);
         }
 
         let lhs_s0 = a.strides()[0];
@@ -245,17 +245,17 @@ fn general_dot_shape_error(m: usize, k: usize, k2: usize, n: usize, c1: usize, c
 /// Return a result array with shape *M*.
 ///
 /// **Panics** if shapes are incompatible.
-impl<A, S, S2> Dot<ArrayBase<S2, Ix>> for ArrayBase<S, Ix2>
+impl<A, S, S2> Dot<ArrayBase<S2, Ix1>> for ArrayBase<S, Ix2>
     where S: Data<Elem=A>,
           S2: Data<Elem=A>,
           A: LinalgScalar,
 {
-    type Output = Array<A, Ix>;
-    fn dot(&self, rhs: &ArrayBase<S2, Ix>) -> Array<A, Ix>
+    type Output = Array<A, Ix1>;
+    fn dot(&self, rhs: &ArrayBase<S2, Ix1>) -> Array<A, Ix1>
     {
         let ((m, a), n) = (self.dim(), rhs.dim());
         if a != n {
-            return dot_shape_error(m, a, n, 1);
+            dot_shape_error(m, a, n, 1);
         }
 
         // Avoid initializing the memory in vec -- set it during iteration
@@ -266,7 +266,7 @@ impl<A, S, S2> Dot<ArrayBase<S2, Ix>> for ArrayBase<S, Ix2>
         for (i, rr) in enumerate(&mut res_elems) {
             unsafe {
                 *rr = (0..a).fold(A::zero(),
-                    move |s, k| s + *self.uget((i, k)) * *rhs.uget(k)
+                    move |s, k| s + *self.uget(Ix2(i, k)) * *rhs.uget(k)
                 );
             }
         }
@@ -312,7 +312,7 @@ fn mat_mul_impl<A>(alpha: A,
 {
     // size cutoff for using BLAS
     let cut = GEMM_BLAS_CUTOFF;
-    let ((mut m, a), (_, mut n)) = (lhs.dim, rhs.dim);
+    let ((mut m, a), (_, mut n)) = (lhs.dim(), rhs.dim());
     if !(m > cut || n > cut || a > cut) ||
         !(same_type::<A, f32>() || same_type::<A, f64>()) {
         return mat_mul_general(alpha, lhs, rhs, beta, c);
@@ -358,8 +358,8 @@ fn mat_mul_impl<A>(alpha: A,
                         }
                     };
                     let n = match rhs_trans {
-                        CblasNoTrans => rhs_.dim().1,
-                        _ => rhs_.dim().0,
+                        CblasNoTrans => rhs_.raw_dim()[1],
+                        _ => rhs_.raw_dim()[0],
                     };
                     // adjust strides, these may [1, 1] for column matrices
                     let lhs_stride = cmp::max(lhs_.strides()[0] as blas_index, k as blas_index);
@@ -403,7 +403,7 @@ fn mat_mul_general<A>(alpha: A,
                       c: &mut ArrayViewMut2<A>)
     where A: LinalgScalar,
 {
-    let ((m, k), (_, n)) = (lhs.dim, rhs.dim);
+    let ((m, k), (_, n)) = (lhs.dim(), rhs.dim());
 
     // common parameters for gemm
     let ap = lhs.as_ptr();
@@ -445,7 +445,7 @@ fn mat_mul_general<A>(alpha: A,
     } else {
         // initialize memory if beta is zero
         if beta.is_zero() {
-            c.assign_scalar(&beta);
+            c.fill(beta);
         }
 
         let mut i = 0;
@@ -489,21 +489,22 @@ pub fn general_mat_mul<A, S1, S2, S3>(alpha: A,
     let ((m, k), (k2, n)) = (a.dim(), b.dim());
     let (m2, n2) = c.dim();
     if k != k2 || m != m2 || n != n2 {
-        return general_dot_shape_error(m, k, k2, n, m2, n2);
+        general_dot_shape_error(m, k, k2, n, m2, n2);
+    } else {
+        mat_mul_impl(alpha, &a.view(), &b.view(), beta, &mut c.view_mut());
     }
-    mat_mul_impl(alpha, &a.view(), &b.view(), beta, &mut c.view_mut());
 }
 
 #[inline(always)]
 /// Return `true` if `A` and `B` are the same type
-fn same_type<A: Any, B: Any>() -> bool {
+fn same_type<A: 'static, B: 'static>() -> bool {
     TypeId::of::<A>() == TypeId::of::<B>()
 }
 
 // Read pointer to type `A` as type `B`.
 //
 // **Panics** if `A` and `B` are not the same type
-fn cast_as<A: Any + Copy, B: Any + Copy>(a: &A) -> B {
+fn cast_as<A: 'static + Copy, B: 'static + Copy>(a: &A) -> B {
     assert!(same_type::<A, B>());
     unsafe {
         ::std::ptr::read(a as *const _ as *const B)
@@ -511,10 +512,10 @@ fn cast_as<A: Any + Copy, B: Any + Copy>(a: &A) -> B {
 }
 
 #[cfg(feature="blas")]
-fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix>) -> bool
+fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix1>) -> bool
     where S: Data,
-          A: Any,
-          S::Elem: Any,
+          A: 'static,
+          S::Elem: 'static,
 {
     if !same_type::<A, S::Elem>() {
         return false;
@@ -533,8 +534,8 @@ fn blas_compat_1d<A, S>(a: &ArrayBase<S, Ix>) -> bool
 #[cfg(feature="blas")]
 fn blas_row_major_2d<A, S>(a: &ArrayBase<S, Ix2>) -> bool
     where S: Data,
-          A: Any,
-          S::Elem: Any,
+          A: 'static,
+          S::Elem: 'static,
 {
     if !same_type::<A, S::Elem>() {
         return false;
